"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function removeFirst(str, toRemove) {
    if (str.indexOf(toRemove) == 0) {
        return str.replace(toRemove, '');
    }
    else {
        return str;
    }
}
var HashHistory = /** @class */ (function () {
    function HashHistory() {
        var _this = this;
        this.handlers = [];
        this.onChange = function () {
            var path = _this.current;
            //console.log('p=', path)
            var redirect = _this.redirects[path];
            if (redirect) {
                _this.replace(redirect);
            }
            else {
                _this.handlers.forEach(function (h) { return h(path); });
            }
        };
        this.redirects = {};
    }
    Object.defineProperty(HashHistory.prototype, "current", {
        get: function () {
            return removeFirst(window.location.hash, '#');
        },
        enumerable: true,
        configurable: true
    });
    HashHistory.prototype.push = function (path) {
        window.location.hash = path;
    };
    HashHistory.prototype.replace = function (path) {
        var href = window.location.href, hashIndex = href.indexOf("#"), newURL = href.slice(0, hashIndex >= 0 ? hashIndex : 0) + "#" + path;
        window.location.replace(newURL);
    };
    HashHistory.prototype.go = function (change) {
        window.history.go(change);
    };
    HashHistory.prototype.goBack = function () {
        this.go(-1);
    };
    HashHistory.prototype.goForward = function () {
        this.go(1);
    };
    HashHistory.prototype.handlerChanged = function () {
        window.removeEventListener('hashchange', this.onChange);
        if (this.handlers.length > 0) {
            window.addEventListener('hashchange', this.onChange);
        }
    };
    HashHistory.prototype.addHandler = function (handler) {
        this.handlers.push(handler);
        this.handlerChanged();
        return this;
    };
    HashHistory.prototype.removeHandler = function (handler) {
        this.handlers = this.handlers.filter(function (h) { return h != handler; });
        this.handlerChanged();
        return this;
    };
    HashHistory.prototype.fire = function () {
        this.onChange();
    };
    return HashHistory;
}());
exports.HashHistory = HashHistory;
//# sourceMappingURL=History.js.map