"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var immer_1 = require("immer");
var React = require("react");
var redux_1 = require("redux");
var react_redux_1 = require("react-redux");
var core_1 = require("./core");
function iterate(obj, keys, func) {
    var skipFields = (obj['_privateFields_'] || []).concat(['onModelInstalled']), filter = function (k) { return !(k.startsWith('_') || includes(skipFields, k) || k == 'constructor'); };
    keys.filter(filter).forEach(function (key) {
        try {
            func(obj[key], key);
        }
        catch (e) {
            console.error(key);
        }
    });
}
/**iterate object*/
function each(obj, func) {
    iterate(obj, Object.keys(obj), func);
}
/**iterate model methods*/
function eachMethod(obj, func) {
    var keys = Object.getOwnPropertyNames(Object.getPrototypeOf(obj));
    iterate(obj, keys, func);
}
function includes(array, item) {
    return array.indexOf(item) != -1;
}
var Effect = /** @class */ (function () {
    function Effect(id, actionType) {
        this.id = id;
        this.actionType = actionType;
        this.children = [];
        this.startTime = 0;
        this.finishTime = 0;
    }
    Effect.prototype.start = function () {
        this.startTime = Date.now();
        this.onStart();
    };
    Effect.prototype.finish = function () {
        this.finishTime = Date.now();
        this.onFinish();
    };
    Effect.prototype.onStart = function () { };
    Effect.prototype.onFinish = function () { };
    Effect.prototype.cancel = function () { };
    return Effect;
}());
var _a = React.createContext(null), UIEventsProvider = _a.Provider, UIEventsConsumer = _a.Consumer;
/**
 * convert class to redux state & action
 * @param cls
 * @param modelId
 */
var ServiceStore = /** @class */ (function () {
    function ServiceStore(models, middleware, reducer) {
        if (middleware === void 0) { middleware = []; }
        var _this = this;
        this.models = models;
        this.dispatcher = {};
        this.computedFields = {};
        this.events = new core_1.UIEvents();
        this.effects = [];
        this.modelHelper = {
            importModel: function (Class) {
                var s = _this, modelId = s.getModelIdByClass(Class), dispatch = s.dispatch[modelId], state = s.store.getState()[modelId], computed = s.computedFields[modelId];
                if (!state) {
                    var modeIds = Object.keys(s.models).join(','), msg = modelId + " is not a valid model ID. Currently installed:" + modeIds + " }";
                    console.error(msg);
                    return null;
                }
                return __assign({}, state, computed, dispatch);
            },
            events: function () {
                return _this.events;
            }
        };
        this.dispatchPool = {};
        this._lastState = null;
        this._lastMergedState = null;
        this.pluginUIs = [];
        this.getService = function (name) {
            return _this.dispatch[name];
        };
        var dispatcher = this.dispatcher, reducers = {}, effectTypes = [], effectFunctions = {}, modelMiddleware = [], pluginIds = [];
        each(models, function (model, modelId) {
            /* for each model, create state, reducers*/
            //create state from fields
            var model_state = {};
            each(model, function (val, key) { return (model_state[key] = val); });
            //create reducer,middleware,effect,calcluated field from methods
            var model_dispatch = (dispatcher[modelId] = {}), //dispatch wrapper, so we can dispatch with method calling
            model_computedFields = (_this.computedFields[modelId] = {}), //calcuated fields
            model_reducers = []; //reducers
            var isPlugin = model instanceof core_1.Plugin;
            eachMethod(model, function (method, methodName) {
                if (method.isComputed) {
                    //calc fields
                    model_computedFields[methodName] = method;
                }
                else if (method.isMiddleware) {
                    // middleware. every middleware is called on every dispatch
                    modelMiddleware.push(function (store) { return function (next) { return function (action) {
                        var type = action.type, isEffect = includes(effectTypes, action.type), isEffectFinish = action.isEffectFinish, isPluginAction = includes(pluginIds, type.split('/')[0]), serviceStore = _this, effectId = action.effectId, effectRootId = _this.getEffectRootId(effectId);
                        var ret = method.call(__assign({}, store.getState()[modelId], model_computedFields, model_dispatch), { store: store, next: next, action: action }, {
                            isEffect: isEffect, isEffectFinish: isEffectFinish, isPluginAction: isPluginAction,
                            model: model, type: type, serviceStore: serviceStore, effectId: effectId, effectRootId: effectRootId
                        });
                        return (ret === undefined) ? next(action) : ret;
                    }; }; });
                }
                else {
                    //effect or reducer
                    var actionType_1 = modelId + "/" + methodName;
                    if (method.isEffect) {
                        //effect
                        effectTypes.push(actionType_1);
                        effectFunctions[actionType_1] = { modelId: modelId, methodName: methodName, method: method };
                    }
                    else {
                        //reducer
                        var reducer_1 = function (state, action) {
                            var match = action.type == actionType_1;
                            return match ? immer_1.default(state, function (draftState) {
                                method.call(draftState, action.payload);
                            }) : state;
                        };
                        model_reducers.push(reducer_1);
                    }
                    //a dispatch method for firing action (effect or reducer)
                    model_dispatch[methodName] = function (payload) {
                        return store.dispatch({ type: actionType_1, payload: payload });
                    };
                    if (method.singleEffect) {
                        var effectMethod_1 = model_dispatch[methodName], singletonM = function (payload) {
                            var _a = method.singleEffect, runAt = _a.runAt, debounce = _a.debounce;
                            var effect = _this.getEffectByActionType(actionType_1);
                            if (effect)
                                effect.cancel();
                            return effectMethod_1(payload);
                        };
                        model_dispatch[methodName] = singletonM;
                    }
                }
            });
            //combine reducers in one model
            reducers[modelId] = function (state, action) {
                return model_reducers.reduce(function (s, reducer) { return reducer(s, action); }, state == undefined ? model_state : state //todo maybe should use init state in createstore
                );
            };
            if (isPlugin) {
                pluginIds.push(modelId);
            }
            if (model['onModelInstalled']) {
                model['onModelInstalled'](_this, model_dispatch, modelId);
            }
        });
        /**
         * the middleware to handle effects
         * @param store
         */
        var effectHandler = function (store) { return function (next) { return function (action) {
            var _a = action, type = _a.type, isEffectFinish = _a.isEffectFinish;
            //to achieve effect cancelling, for example, a effect does ajax then calls a reducer.
            //you can cancel it when it's running , the reducer will be blocked
            if (action.effectId) {
                if (!_this.hasEffect(action.effectId)) {
                    throw action.type + " has been cancelled";
                }
            }
            //pass if it is a reducer
            if (!includes(effectTypes, type))
                return next(action);
            //this is a 'finishing' action, effect just finished
            if (isEffectFinish) {
                //console.log('effectHandler closing...', action.type) 
                var effect_1 = _this.getEffectById(action.effectId);
                if (effect_1) {
                    //console.log(effect)
                    effect_1.finish();
                }
                else {
                    console.log('cannot find when close', action);
                }
                //console.log(this.effects)
                return action.payload;
            }
            //handle effect
            var _b = effectFunctions[type], modelId = _b.modelId, method = _b.method;
            //start an effect
            var effect = _this.createEffect(type, action.effectId), effectId = action.effectId = effect.id;
            var effectDispatch = {};
            Object.keys(_this.dispatcher[modelId]).forEach(function (methodName) {
                effectDispatch[methodName] = function (payload) {
                    return store.dispatch({
                        type: modelId + "/" + methodName,
                        payload: payload, effectId: effectId
                    });
                };
            });
            var state = store.getState(), modelState = state[modelId], computed = _this.computedFields[modelId], helper = models[modelId] instanceof core_1.Model ? _this.modelHelper : null, mixedContext = __assign({}, modelState, computed, effectDispatch, helper);
            effect.start();
            var ret = method.call(mixedContext, action.payload, state);
            next(action);
            //use Promise.resolve incase effect returns non promise
            return Promise.resolve(ret).then(function (payload) {
                return store.dispatch({
                    type: type, payload: payload, effectId: effectId,
                    isEffectFinish: true
                });
            }).catch(function (reason) {
                console.log(reason);
            });
        }; }; };
        var combinedReducer = redux_1.combineReducers(reducers);
        if (reducer) {
            //compose
            var _reducer_1 = combinedReducer;
            combinedReducer = function (state, action) {
                return reducer(_reducer_1(state, action), action);
            };
        }
        var composeEnhancers = window['__REDUX_DEVTOOLS_EXTENSION_COMPOSE__'] || redux_1.compose;
        middleware = modelMiddleware.concat([effectHandler]).concat(middleware);
        var store = this.store = redux_1.createStore(combinedReducer, composeEnhancers(redux_1.applyMiddleware.apply(void 0, middleware)));
    }
    ServiceStore.prototype.getEffect = function (match) {
        function search(effects) {
            var find = null;
            effects.some(function (c) {
                if (match(c)) {
                    find = c;
                    return true;
                }
                else {
                    find = search(c.children);
                    return find != null;
                }
            });
            return find;
        }
        return search(this.effects);
    };
    ServiceStore.prototype.getEffectById = function (id) {
        return this.getEffect(function (effect) { return effect.id == id; });
    };
    ServiceStore.prototype.getEffectByActionType = function (actionType) {
        return this.getEffect(function (effect) { return effect.actionType == actionType; });
    };
    ServiceStore.prototype.removeEffect = function (id) {
        var effect = this.getEffectById(id);
        if (effect) {
            if (effect.parent) {
                effect.parent.children = effect.parent.children.filter(function (c) { return c != effect; });
            }
            this.effects = this.effects.filter(function (e) { return e !== effect; });
        }
    };
    ServiceStore.prototype.getEffectRoot = function (id) {
        var effect = this.getEffectById(id);
        while (effect && effect.parent) {
            effect = effect.parent;
        }
        return effect;
    };
    ServiceStore.prototype.getEffectRootId = function (id) {
        var root = this.getEffectRoot(id);
        return root ? root.id : null;
    };
    ServiceStore.prototype.hasEffect = function (id) {
        return this.getEffectById(id) != null;
    };
    ServiceStore.prototype.createEffect = function (actionType, parentId) {
        var _this = this;
        var effectId = Date.now() + (Math.random() + '').substring(2), effect = new Effect(effectId, actionType);
        if (parentId) {
            var parent_1 = this.getEffectById(parentId);
            if (parent_1) {
                parent_1.children.push(effect);
                effect.parent = parent_1;
            }
        }
        else {
            this.effects.push(effect);
        }
        effect.onFinish = function () {
            _this.removeEffect(effect.id);
        };
        effect.cancel = function () {
            _this.store.dispatch({
                type: actionType, payload: null, effectId: effectId,
                isEffectFinish: true
            });
        };
        return effect;
    };
    /**
     * return the targetting model of the action
     * @param actionType
     */
    ServiceStore.prototype.getModelByActionType = function (actionType) {
        var modelId = actionType.split('/')[0];
        return this.models[modelId];
    };
    ServiceStore.prototype.getModelIdByClass = function (actionType) {
        var modelId = null;
        each(this.models, function (val, key) {
            var match = val instanceof actionType;
            if (match)
                modelId = key;
        });
        return modelId;
    };
    Object.defineProperty(ServiceStore.prototype, "dispatch", {
        get: function () {
            return this.dispatcher;
        },
        enumerable: true,
        configurable: true
    });
    ServiceStore.prototype.getEffectDispatch = function (effectId) {
        var dispatchPool = this.dispatchPool;
        if (!dispatchPool[effectId]) {
            var effectDispatch_1 = (dispatchPool[effectId] = {}), _a = this, dispatcher = _a.dispatcher, store_1 = _a.store;
            each(dispatcher, function (modelDispatch, modelId) {
                var modelEffectDispatch = (effectDispatch_1[modelId] = {});
                each(modelDispatch, function (method, methodName) {
                    modelEffectDispatch[methodName] = function (payload) {
                        var type = modelId + "/" + methodName;
                        store_1.dispatch({ type: type, payload: payload, effectId: effectId });
                    };
                });
            });
        }
        return dispatchPool[effectId];
    };
    Object.defineProperty(ServiceStore.prototype, "state", {
        /**
         * root state, doesn't feel very useful?
         */
        get: function () {
            return this.store.getState();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServiceStore.prototype, "connect", {
        /**
         * connect decorator to use with React.Component.
         * it maps state (or calcudated fields) to component props
         */
        get: function () {
            var _this = this;
            var connect = function (stateMap //decorator factory
            ) { return function (method) { return react_redux_1.connect(function () { return stateMap(_this.getMergedState()); })(method); }; };
            return connect;
        },
        enumerable: true,
        configurable: true
    });
    ServiceStore.prototype.getMergedState = function () {
        var _this = this;
        var state = this.state;
        if (state == this._lastState)
            return this._lastMergedState; //use cache
        var mergedState = {};
        Object.keys(state).forEach(function (modelId) {
            var modelState = state[modelId], computed = _this.computedFields[modelId], helper = _this.models[modelId] instanceof core_1.Model ? _this.modelHelper : null;
            mergedState[modelId] = __assign({}, modelState, computed, helper);
        });
        this._lastState = state;
        this._lastMergedState = mergedState;
        return mergedState;
    };
    Object.defineProperty(ServiceStore.prototype, "Provider", {
        get: function () {
            var _this = this;
            var _a = this, store = _a.store, pluginUIs = _a.pluginUIs;
            return function (props) {
                var pluginEles = pluginUIs.map(function (ui, i) { return React.createElement(ui, { key: i }); }), propChildren = React.Children.toArray(props.children), wrapper = React.createElement(React.Fragment, {}, pluginEles.concat(propChildren));
                return React.createElement(UIEventsProvider, { value: _this.events }, React.createElement(react_redux_1.Provider, { store: store }, wrapper));
            };
        },
        enumerable: true,
        configurable: true
    });
    return ServiceStore;
}());
function createStore(models, middlewares, reducer) {
    return new ServiceStore(models, middlewares, reducer);
}
exports.createStore = createStore;
function withUIEvents(Component) {
    return function Wrapper(props) {
        return (React.createElement(UIEventsConsumer, null, function (events) { return React.createElement(Component, __assign({}, props, { events: events })); }));
    };
}
exports.withUIEvents = withUIEvents;
//# sourceMappingURL=ServiceStore.js.map