"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var Transition = /** @class */ (function (_super) {
    __extends(Transition, _super);
    function Transition() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            components: [],
            props: null
        };
        _this.timers = [];
        return _this;
    }
    Transition.getDerivedStateFromProps = function (props, state) {
        if (props == state.props)
            return state;
        //compare prop to state comp list, new->enter, missing->leave
        var oldComps = state.components.map(function (info) { return info.comp; }), newComps = props.component ? [props.component] : React.Children.toArray(props['children']);
        var skip = (oldComps.length == 0) && (!props.transFirst), sampeComp = (oldComps.length == 1 && newComps.length == 1 && oldComps[0] == newComps[0]);
        var newCompInfos = newComps.map(function (newComp) {
            var existing = oldComps.some(function (co) { return co == newComp; });
            return {
                comp: newComp,
                action: existing || skip || sampeComp ? '' : 'WillEnter'
            };
        });
        var toLeave = oldComps.filter(function (c) { return newComps.indexOf(c) == -1; }).map(function (c) { return ({
            comp: c,
            action: 'WillLeave'
        }); });
        return {
            components: newCompInfos.concat(toLeave),
            props: props
        };
    };
    Transition.prototype.render = function () {
        var _this = this;
        var components = this.state.components, _a = this.props, getClassName = _a.getClassName, injectClass = _a.injectClass;
        return React.createElement(React.Fragment, null, components.map(function (c, i) {
            var className = getClassName(c.action);
            return (injectClass) ? (React.cloneElement(c.comp, __assign({}, c.comp.props, { className: className, key: i }))) : React.createElement("div", { key: i, style: {
                    transitionDuration: (_this.props.transitionDuration) + 'ms'
                }, className: getClassName(c.action) }, c.comp);
        }));
    };
    Transition.prototype.startAnimation = function () {
        var components = this.state.components, willEnters = components.filter(function (c) { return c.action == 'WillEnter'; }), willLeaves = components.filter(function (c) { return c.action == 'WillLeave'; });
        if (willEnters.length + willLeaves.length) {
            willEnters.forEach(function (c) { return c.action = 'Enter'; });
            willLeaves.forEach(function (c) { return c.action = 'Leave'; });
            this.setState({ components: components.slice() });
        }
    };
    Transition.prototype.fnishAnimation = function () {
        var components = this.state.components, enters = components.filter(function (c) { return c.action == 'Enter'; }), leaves = components.filter(function (c) { return c.action == 'Leave'; });
        if (enters.length + leaves.length) {
            var newComp = components.filter(function (c) { return leaves.indexOf(c) == -1; });
            newComp.forEach(function (c) { return c.action = ''; });
            this.setState({ components: newComp });
        }
    };
    Transition.prototype.clearTimer = function () {
        var timers = this.timers;
        if (timers.length) {
            timers.forEach(function (t) { return clearTimeout(t); });
            timers.length = 0;
        }
        return timers;
    };
    Transition.prototype.componentDidUpdate = function () {
        this.checkAnimation();
    };
    Transition.prototype.componentDidMount = function () {
        this.checkAnimation();
    };
    Transition.prototype.checkAnimation = function () {
        var _this = this;
        var components = this.state.components;
        if (components.some(function (c) { return c.action.startsWith('Will'); })) {
            var timers = this.clearTimer();
            timers.push(setTimeout(function () {
                _this.startAnimation();
            }, 10));
            timers.push(setTimeout(function () {
                _this.fnishAnimation();
            }, this.props.transitionDuration));
        }
    };
    Transition.prototype.componentWillUnmount = function () {
        this.clearTimer();
    };
    Transition.defaultProps = {
        transitionDuration: 300,
        getClassName: function (action) { return action; },
        transFirst: false,
        injectClass: false
    };
    return Transition;
}(React.PureComponent));
exports.Transition = Transition;
//# sourceMappingURL=Transition.js.map