import './polyfill';
interface Action {
    type: string;
    payload: any;
    effectId?: string;
    isEffectFinish?: boolean;
}
/** effects handles side effects/async tasks, the method needs to be async  */
declare function effect(target: Object, key: string, desc: TypedPropertyDescriptor<(payload?: any) => Promise<any>>): void;
declare function singleEffect(target: Object, key: string, desc: TypedPropertyDescriptor<(payload?: any) => Promise<any>>): void;
/** computed fields can be used in 'connect', it output values based on state  */
declare function computed(target: Object, key: string, desc: TypedPropertyDescriptor<() => any>): void;
/** tag a method a middleware, can only be used on method of a Plugin class  */
declare function middleware(target: Plugin, key: string, desc: TypedPropertyDescriptor<(ctx: middleware.Context, info: middleware.Info) => any>): void;
declare module middleware {
    interface Context {
        store: any;
        next: Function;
        action: Action;
    }
    interface Info {
        type: string;
        isEffect: boolean;
        isEffectFinish: boolean;
        isPluginAction: boolean;
        effectId: string;
        effectRootId: string;
        model: any;
        serviceStore: any;
    }
}
/**
 * Model class provides helper methods like getSibling.
 * It's not mandatory for a model to extend this class, unless it needs those methods
 */
declare class Model {
    /**
     * fetch sibling model for reading its state or calling its reducer/effect
     * @param modelId
     */
    protected importModel<T>(Class: new () => T): T;
    events(): UIEvents;
    /** trigged when model is installed  */
    protected onModelInstalled(storeSore: any, dispatch: any, modelId: string): void;
}
/**
 * Implement Plugin to provide common functions like logging, displaying loading bar
 * Plugin class doesn't provide any extra function than Model class, at the moment
 */
declare class Plugin extends Model {
    _isPlugin_: boolean;
}
/**
 * the UI Events handls small user interaction, like alert or prompt.
 * service store contains a single events.
 */
declare class UIEvents {
    private pool;
    run(onStart: (id: string) => void, onFinish?: (ret: any) => void): Promise<{}>;
    private createUniqId;
    solve(pid: any, ret: any): void;
    readonly count: number;
    private notify;
    private observers;
    addObserver(ob: (brokder: UIEvents) => void): void;
}
export { effect, singleEffect, computed, middleware, Action, Model, Plugin, UIEvents };
